% Lab 
% HG 
% Yaw control for the Tello drone
% P and PD design from the linear model identified from experimental 
% squave wave response data 

close all
clc
clear
format compact

% Vertical dynamic model parameters identified from experimental step response
K=0.80; % Adjust according to your model
T=0.19; % Adjust according to your model

r=30; % Amplitude of the yaw setpoint in °

% Desired performance in closed loop
D1pourcent=4.3;
ts5=0.5; % Desired settling time in closed loop

% Sampling time for the closed loop control
Ts=0.25;

%% P-control

D1=D1pourcent/100;

z=sqrt(log(D1)^2/(log(D1)^2+pi^2))
%w0=3/ts5;
Kp=1/(4*z^2*K*T)

%Kp=1/(2*K*T) if z=sqrt(2)/2;


w0=2*z*K*Kp % No choice to impose w0 with P controller
% w0=sqrt(K*Kp/T) same result than above

Kpp=Kp;

[N,D]=ord2(w0,z);
F_CL=tf(w0^2,D)
%F_CLbis=tf(K*Kp/T,[1 1/T K*Kp/T])

[y1,t]=step(r*F_CL,[0:0.001:5]');
N=length(y1);
ref=r*[0;ones(N-1,1)];
plot(t,ref,'b--',t,y1,'-r','linewidth',2)
ylabel('Drone vertical position (in cm)')
xlabel('Time (s)')
title(['P servo control when K_p=',num2str(Kp,'% 10.2f')])
set(gca,'FontSize',14,'FontName','helvetica');
axis([0 t(end) 0 r+10])
grid
shg
stepinfo(r*F_CL,'SettlingTimeThreshold',0.05,'RiseTimeLimits',[0 0.95])

Peak_Time_P=pi/w0/(sqrt(1-z^2))

%% PD controller with the Derivative effect on the output

%w0_PD=3/(z*ts5)
w0_PD=3/ts5

Kp=T*w0_PD^2/K
Kd=2*z*Kp/(w0_PD)-1/K

figure
[N,D]=ord2(w0_PD,z);
F_CL=tf(w0_PD^2,D);

[y2,t]=step(r*F_CL,t);
N=length(y1);
ref=r*[0;ones(N-1,1)];
plot(t,ref,'b--',t,y2,'-r','linewidth',2)
ylabel('Yaw angle (in degree)')
xlabel('Time (s)')
title(['PD servo control when K_p=',num2str(Kp,'% 10.1f'),' and K_d=',num2str(Kd,'% 10.2f')])
set(gca,'FontSize',14,'FontName','helvetica');
axis([0 t(end) 0 r+10])
grid
shg
stepinfo(r*F_CL,'SettlingTimeThreshold',0.05,'RiseTimeLimits',[0 0.95])

Peak_Time_PD=pi/w0_PD/(sqrt(1-z^2))
figure
[N,D]=ord2(w0_PD,z);
F_CL=tf(w0_PD^2,D);
[y2,t]=step(r*F_CL,t);
N=length(y1);
plot(t,ref,'b--',t,y1,'-r',t,y2,'-.k','linewidth',2)
ylabel('Yaw angle (in degree)')
xlabel('Time (s)')
legend('Setpoint','P servo control','PD servo control','location','Southeast')
title(['P control K_p=',num2str(Kpp,'% 10.2f'),' - PD control K_p=',num2str(Kp,'% 10.1f'),' and K_d=',num2str(Kd,'% 10.2f')])
set(gca,'FontSize',14,'FontName','helvetica');
axis([0 t(end) 0 r+10])
grid
shg
