from djitellopy import Tello  # Import Tello class from the djitellopy library
import time  # Import time module for time-based functions
import matplotlib.pyplot as plt  # Import matplotlib for plotting data
import concurrent.futures  # Import for concurrent execution
import threading  # Import threading for multi-threading

import datetime # To monitor time
#now = datetime.datetime.now()

# Global variables
Ts = 0.015  # Sampling period in seconds
RC_Z = RC_YAW = 0  # Initialize remote control commands for altitude and yaw
seuil = 5  # Threshold value (not used in this snippet)
Z_ref = YAW_ref = 0  # Initialize setpoint variables for altitude and yaw

VRR = 100

# Control flags
Validation = False
Registering = True
Controling = True

# Filenames for data storage
file_name = "dataCL_yaw_Tello.txt"
file_name_sampling = "dataCL_Tello_sampling.txt"


#################################
##          Functions          ##
#################################

def check(delay):
    global Validation
    while Controling == True:
        time.sleep(delay)
        Validation = True

def go_to_position(waypoints):
    """Function to control the drone's position by sending yaw commands based on proportional control."""

    global RC_YAW, YAW_ref, Controling, Validation
    kp = 2.5  # Proportional gain, adjust as needed
    kd = 0 # Derivative gain, to be set


    Last_Yaw = 0 # Yaw at time (k-1)
    
    point_index = 0

    while point_index < len(waypoints):
            ############ Z ############
        YAW_ref = waypoints[point_index][3]  # Desired yaw from waypoints
        YAW = tello.get_yaw()  # Measured yaw
        e_yaw = YAW_ref - YAW  # Yaw error

        U_kp = kp * e_yaw  # Calculate proportional control
        U_kd = 0           # Calculate derivative control
        rc_yaw = U_kp - U_kd

        rc_yaw = int(saturation(rc_yaw, -100, 100))  # Apply saturation to the control command

        Last_Yaw = YAW
        RC_YAW = rc_yaw  # Update the global control command
    

            ###########################

        if Validation == True:
            point_index += 1
            Validation = False

        time.sleep(Ts)
    
def send_rc():
    """Function to continuously send control commands to the drone."""
    while Controling == True:
        tello.send_rc_control(0, 0, RC_Z, RC_YAW)
        time.sleep(Ts)

def saturation(val, min, max):
    if val > max:
        val = max
    elif val < min:
        val = min
    return val

def register(start_time):
    print("Registering\n")
    result = open(file_name,"w")
    global RC_YAW, YAW_ref

    while Registering == True:
        instant_time = time.time() - start_time
 
        result.write(str(instant_time) + " " + str(tello.get_distance_tof()) + " " + str(RC_YAW) + " " + str(YAW_ref) + " " + str(tello.get_yaw()) +"\n")  
        time.sleep(Ts)
    
    result.close()
    
    print("Done registering")



##############################
##          Control         ##
##############################



tello = Tello()
tello.connect()

print("battery: " + str(tello.get_battery()) + "%")

time.sleep(3)

######################################
##      Closed Loop - Beginning     ## 

with concurrent.futures.ThreadPoolExecutor() as executor:
    delay = 5 # Period of the input step signal in seconds
    start_time = time.time()
    
    executor.map(register, [start_time])  

    tello.takeoff()

    time.sleep(3)

    waypoints = [[0, 0, 0, 30],
                 [0, 0, 0, 0],
                 [0, 0, 0, -30],
                 [0, 0, 0, 0],
                 [0, 0, 0, 30],
                 [0, 0, 0, 0]] ## Input steps


    executor.submit(send_rc)
    executor.map(check, [delay])
    
    go_to_position(waypoints)
    

    Controling = False
    running_check = False

    time.sleep(1)

    tello.send_rc_control(0,0,0,0)



    time.sleep(3)
    
    RC_Z = 0
    time.sleep(1)
    tello.land()

    Registering = False

##      Closed Loop - End           ## 
######################################

############################
##          Plot          ##
############################

time.sleep(1) 

fichier = open(file_name,"r")


Time = []
Altitude = []
cmd = []
consign_yaw = []
yaw = []
for ligne in fichier:
    valeurs = ligne.split()
    Time.append(float(valeurs[0]))
    Altitude.append(float(valeurs[1]))
    cmd.append(float(valeurs[2]))
    consign_yaw.append(float(valeurs[3]))
    yaw.append(float(valeurs[4]))

fichier.close()

fig, (ax1,ax2) = plt.subplots(2,1)
#ax1.plot(Time, Altitude)
ax1.plot(Time, yaw)
ax1.plot(Time, consign_yaw)
ax1.legend(["Yaw angle (°)","Yaw setpoint (°)"])

ax2.plot(Time, cmd)
ax2.legend(["Motor speed (%)"])
plt.xlabel('Time (s)')

plt.show()