from djitellopy import Tello
import time
import matplotlib.pyplot as plt
import concurrent.futures
import threading

Ts = 0.015
RC_Z = RC_YAW = 0
seuil = 5
Z = YAW = 0

VRR = 100

Validation = False
Controling = True

running_check = False



#################################
##          Functions          ##
#################################



def check():
    global Validation, running_check
    running_check = True
    while running_check:
        if abs(VRR) < seuil:      
            st = time.time()
            while (time.time() - st) < 2:
                Temporary_validation = False
                if abs(VRR) < seuil:
                    Temporary_validation = True
                else:
                    break
            Validation = Temporary_validation
        time.sleep(Ts)  # run check in every Ts

def go_to_position(waypoints):
    """Go to z coordinates and check that Z is around the wanted value"""

    print("going to position")

    global RC_Z, VRR, Z, Controling, Validation

    
    point_index = 0

    thread = None

    if thread is None:  # Only start a new thread if the old one is finished
        thread = threading.Thread(target=check)
        thread.start()

    while point_index < len(waypoints):
            ############ Z ############

        rc_z = 0

        rc_z = int(saturation(rc_z, -100, 100))

        RC_Z = rc_z
        
        #VRR = e_z

        if Validation == True:
            point_index += 1
            Validation = False

        time.sleep(Ts)
  
def send_rc():
    while Controling == True:
        tello.send_rc_control(0, 0, RC_Z, RC_YAW)
        time.sleep(Ts)

def saturation(val, min, max):
    if val > max:
        val = max
    elif val < min:
        val = min
    return val

def register(start_time):
    print("Registering\n")
    DataOL_Tello = open("DataOL_yaw_Tello.txt","w")
    global RC_Z, Z

    while Registering == True:
        instant_time = time.time() - start_time
 
        DataOL_Tello.write(str(instant_time) + " " + str(tello.get_distance_tof()) + " " + str(RC_Z) + " " + str(Z) + " " + str(tello.get_yaw()) +"\n")        
        time.sleep(Ts)
    
    DataOL_Tello.close()
    
    print("Done registering")



##############################
##          Control         ##
##############################



tello = Tello()
tello.connect()

print("battery: " + str(tello.get_battery()) + "%")

time.sleep(3)

######################################
##      Open Loop - Beginning       ##
with concurrent.futures.ThreadPoolExecutor() as executor:
    start_time = time.time()
    Registering = True

    executor.map(register, [start_time])  

    tello.takeoff()

    time.sleep(3)

    Controling = False
    running_check = False

    ####       Write your set of step commands below       ###

    # The code below controls in open loop the Tello drone by sending vertical control commands (up or down) 
    # with varying amplitudes and then pausing for intervals of 3 seconds between commands. 
    # The sequence alternates between a positive amplitude (up), a zero amplitude (hover), a negative amplitude (down), 
    # and then back to zero amplitude, repeated 5 times.

    ###############################
    amplitude = 50

    time.sleep(3)

    # Define the sequence of RC_Z values
    rc_z_values = [amplitude, 0, -amplitude, 0] * 3  # Repeat the pattern 5 times

    for RC_Z in rc_z_values:
        # Send the RC control command with the current RC_Z value
        # tello.send_rc_control(0, 0, RC_Z, 0)
        tello.send_rc_control(0, 0, 0, RC_Z)
        # Pause for 3 seconds before sending the next command
        time.sleep(3)
    ###############################
    
    time.sleep(2)

    
    tello.land()

    Registering = False

##      Open Loop - End             ## 
######################################



############################
##          Plot          ##
############################

fichier = open("DataOL_yaw_Tello.txt","r")

Time = []
Altitude = []
cmd = []
consign = []
yaw = []
for ligne in fichier:
    valeurs = ligne.split()
    Time.append(float(valeurs[0]))
    Altitude.append(float(valeurs[1]))
    cmd.append(float(valeurs[2]))
    consign.append(float(valeurs[3]))
    yaw.append(float(valeurs[4]))

fichier.close()


fig, (ax1,ax2,ax3) = plt.subplots(3,1)
ax1.plot(Time, Altitude)
ax1.legend(["Altitude (in cm)"])

ax2.plot(Time, yaw)
ax2.legend(["Yaw angle (in °)"])

ax3.plot(Time, cmd)
plt.xlabel('Time (s)')
ax3.legend(["Motor speed (in %)"])

plt.show()

    